import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IVariableClassType } from '../variable-class-type.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../variable-class-type.test-samples';

import { VariableClassTypeService } from './variable-class-type.service';

const requireRestSample: IVariableClassType = {
  ...sampleWithRequiredData,
};

describe('VariableClassType Service', () => {
  let service: VariableClassTypeService;
  let httpMock: HttpTestingController;
  let expectedResult: IVariableClassType | IVariableClassType[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(VariableClassTypeService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a VariableClassType', () => {
      const variableClassType = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(variableClassType).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a VariableClassType', () => {
      const variableClassType = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(variableClassType).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a VariableClassType', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of VariableClassType', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a VariableClassType', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addVariableClassTypeToCollectionIfMissing', () => {
      it('should add a VariableClassType to an empty array', () => {
        const variableClassType: IVariableClassType = sampleWithRequiredData;
        expectedResult = service.addVariableClassTypeToCollectionIfMissing([], variableClassType);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(variableClassType);
      });

      it('should not add a VariableClassType to an array that contains it', () => {
        const variableClassType: IVariableClassType = sampleWithRequiredData;
        const variableClassTypeCollection: IVariableClassType[] = [
          {
            ...variableClassType,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addVariableClassTypeToCollectionIfMissing(variableClassTypeCollection, variableClassType);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a VariableClassType to an array that doesn't contain it", () => {
        const variableClassType: IVariableClassType = sampleWithRequiredData;
        const variableClassTypeCollection: IVariableClassType[] = [sampleWithPartialData];
        expectedResult = service.addVariableClassTypeToCollectionIfMissing(variableClassTypeCollection, variableClassType);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(variableClassType);
      });

      it('should add only unique VariableClassType to an array', () => {
        const variableClassTypeArray: IVariableClassType[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const variableClassTypeCollection: IVariableClassType[] = [sampleWithRequiredData];
        expectedResult = service.addVariableClassTypeToCollectionIfMissing(variableClassTypeCollection, ...variableClassTypeArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const variableClassType: IVariableClassType = sampleWithRequiredData;
        const variableClassType2: IVariableClassType = sampleWithPartialData;
        expectedResult = service.addVariableClassTypeToCollectionIfMissing([], variableClassType, variableClassType2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(variableClassType);
        expect(expectedResult).toContain(variableClassType2);
      });

      it('should accept null and undefined values', () => {
        const variableClassType: IVariableClassType = sampleWithRequiredData;
        expectedResult = service.addVariableClassTypeToCollectionIfMissing([], null, variableClassType, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(variableClassType);
      });

      it('should return initial array if no VariableClassType is added', () => {
        const variableClassTypeCollection: IVariableClassType[] = [sampleWithRequiredData];
        expectedResult = service.addVariableClassTypeToCollectionIfMissing(variableClassTypeCollection, undefined, null);
        expect(expectedResult).toEqual(variableClassTypeCollection);
      });
    });

    describe('compareVariableClassType', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareVariableClassType(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareVariableClassType(entity1, entity2);
        const compareResult2 = service.compareVariableClassType(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareVariableClassType(entity1, entity2);
        const compareResult2 = service.compareVariableClassType(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareVariableClassType(entity1, entity2);
        const compareResult2 = service.compareVariableClassType(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
